@@ -48,6 +48,9 @@ use Sys::Syslog qw ( :DEFAULT setlogsock );
 
 $DEBUG=0;
 
+$SUCCESS=1;
+$FAILURE=2;
+
 ############# User Configurable options.
 
 $WEBSERVER="/usr/sbin/SnareWebServer.pl";
@@ -79,7 +82,7 @@ $SYSLOGDEST=0;
 
 ############# End user configurable options.
 
-$VERSION="1.1";
+$VERSION="1.2";
 
 $FLUSH=0;
 $FLUSHNOW=-1;
@@ -398,40 +401,71 @@ sub SendEvents {
 		delete $event{$eventnum};
 
 		# Format of the filters:
-		#$Filters{$ObjectiveCount}{$key}{$ElementCount}{$AlternativeCount}=$alternative;
-		#$Filters{0}{event}{0}{0}="execve";
+		#$Filters{$ObjectiveCount}{$key}{$keymatch}{$ElementCount}{$AlternativeCount}=$alternative;
+		#$Filters{0}{event}{0}{0}{0}="execve";
 		$EventMatched=0;
 		$EventCrit=0;
 		foreach $objectivecount (keys(%Filters)) {
 			$ObjectiveMatch=0;
 			foreach $key (keys(%{$Filters{$objectivecount}})) {
 				if($tokens{$key}) {
-					foreach $elementcount (keys(%{$Filters{$objectivecount}{$key}})) {
-						@elements=split(/,/,$tokens{$key});
-						if($DEBUG>3) { LogMsg("Key: $key - ELEMENTS: @elements\n"); }
-						$result=0;
-						foreach $alternativecount (keys(%{$Filters{$objectivecount}{$key}{$elementcount}})) {
-							$match=$Filters{$objectivecount}{$key}{$elementcount}{$alternativecount};
-							#$element=$elements[$elementcount];
-							$element=$elements[0];
-							$negate=$FilterTypes{$objectivecount}{$key};
-							if($DEBUG>2) { LogMsg("Match element $element against $match (negate is $negate)\n"); }
+					foreach $keymatch (keys(%{$Filters{$objectivecount}{$key}})) {
+						foreach $elementcount (keys(%{$Filters{$objectivecount}{$key}{$keymatch}})) {
+							@elements=split(/,/,$tokens{$key});
+							if($DEBUG>3) { LogMsg("Key: $key - ELEMENTS: @elements\n"); }
+							$result=0;
+							foreach $alternativecount (keys(%{$Filters{$objectivecount}{$key}{$keymatch}{$elementcount}})) {
+								$match=$Filters{$objectivecount}{$key}{$keymatch}{$elementcount}{$alternativecount};
+								$element=$elements[$elementcount];
+								$negate=$FilterTypes{$objectivecount}{$key}{$keymatch};
+								if($DEBUG>2) { LogMsg("Match element $element against $match (negate is $negate)\n"); }
 
-							# Special case:
-							if($match eq "*") {
-								$result=1;
-							} else {
-								$result=Match($element,$match,$negate);
+								# Special case:
+								if($match eq "*") {
+									$result=1;
+								} else {
+									@strings=split(/,/,$element);
+									@matches=split(/,/,$match);
+									$count=0;
+									foreach $tmatch (@matches) {
+										if ($REGEX == 1) {
+											$regex=$tmatch;
+											$regex =~ s/[^\\]\//\\\//g;
+										} else {
+											#$regex=wildcard($tmatch);
+											my $re = quotemeta $tmatch;
+											$re =~ s/\\\*/(.*)/g;
+											$regex = "^" . $re . "\$";
+										}
+										if($DEBUG>1) { LogMsg("Match: $regex against " . $strings[$count]); }
+										if($strings[$count] =~ /$regex/) {
+											$result=!$negate;
+										} else {
+											$result=$negate;
+										}
+										if(!$result) {
+											if($DEBUG > 1) { LogMsg("Match: No Match! (negate is $negate)"); }
+											break;
+										}
+										$count++;
+									}
+									#$result=Match($element,$match,$negate);
+								}
+								#if($result && !$negate || !$result && $negate) {}
+								if($result) {
+									# Yay - one of the alternatives matched
+									if($DEBUG > 1) { LogMsg("BREAKING out (result is $result, negate is $negate)"); }
+									last;
+								}
 							}
-							if($result && !$negate || !$result && $negate) {
-								# Yay - one of the alternatives matched
+							if($result) {
+								$ObjectiveMatch=1;
+							} else {
+								$ObjectiveMatch=0;
 								last;
 							}
 						}
-						if($result) {
-							$ObjectiveMatch=1;
-						} else {
-							$ObjectiveMatch=0;
+						if(!$ObjectiveMatch) {
 							last;
 						}
 					}
@@ -452,7 +486,7 @@ sub SendEvents {
 						$EventCrit=$tcrit;
 					}
 				} else {
-					$EventCrit=0;
+					$EventCrit=$Criticality[$objectivecount];
 					last;
 				}
 			}
@@ -494,7 +528,6 @@ sub SendEvent() {
 				# Prepend some syslog gumf. REDRED TODO
 				$sdate=strftime("%b %e %H:%M:%S",localtime);
 				$sprefix="<$SYSLOGDEST> $sdate $FQDN LinuxKAudit";
-				#DMM
 				#$sprefix="<$SYSLOGDEST> LinuxKAudit";
 				send($OUTPUTNET{$i}{"Socket"}, "$sprefix\t$string", 0, $OUTPUTNET{$i}{"PortAddr"});
 			} else {
@@ -594,7 +627,8 @@ sub LoadConfig() {
 				}
 			} elsif($section eq "OBJECTIVES") {
 				# Default value.
-				$Criticality[$ObjectiveCount]=0;
+				$TEMPObjectiveCount = $ObjectiveCount;
+				$Criticality[$TEMPObjectiveCount]=0;
 
 				@elements=split(/\t/,$line);
 				%Config=();
@@ -602,7 +636,7 @@ sub LoadConfig() {
 				foreach $element (@elements) {
 					($key,$check,$val)=split(/(\!*=)/,$element);
 					if($key eq "criticality") {
-						$Criticality[$ObjectiveCount]=$val;
+						$Criticality[$TEMPObjectiveCount]=$val;
 						next;
 					}
 
@@ -621,29 +655,45 @@ sub LoadConfig() {
 					}
 					
 					$ElementCount=0;
+					$keymatch=keys(%{$Filters{$TEMPObjectiveCount}{$key}});
 					foreach $component (@components) {
 						$AlternativeCount=0;
 						foreach $alternative (split(/,/,$component)) {
-			if($DEBUG) { LogMsg("alternative $key-$ElementCount.$AlternativeCount:$alternative\n"); }
-							$Filters{$ObjectiveCount}{$key}{$ElementCount}{$AlternativeCount}=$alternative;
+							#LogMsg("Adding $alternative to $TEMPObjectiveCount:$key:$keymatch:$ElementCount:$AlternativeCount");
+							$Filters{$TEMPObjectiveCount}{$key}{$keymatch}{$ElementCount}{$AlternativeCount}=$alternative;
 							if($check eq "=") {
 								# If the user has specified a non-negate match, and
 								# Has identified an explicit event, then mark it to be turned on.
 								if($key eq "event") {
-									$EventsON{$alternative}=1;
+									# put the events in a temporary list
+									$temp_EventsON{$alternative}=$SUCCESS + $FAILURE;
+								} elsif($key eq "return") {
+									# if we are only looking for a specific return code,
+									# only enable the necessary auditing
+									if ($alternative eq "no") {
+										foreach $eventid (keys(%temp_EventsON)) {
+											$temp_EventsON{$eventid}=$FAILURE;
+										}
+									} elsif ($alternative eq "yes") {
+										foreach $eventid (keys(%temp_EventsON)) {
+											$temp_EventsON{$eventid}=$SUCCESS;
+										}
+									}
 								}
-								$FilterTypes{$ObjectiveCount}{$key}=0;
+								$FilterTypes{$TEMPObjectiveCount}{$key}{$keymatch}=0;
 							} else {
 								# Negate this match
-								$FilterTypes{$ObjectiveCount}{$key}=1;
+								$FilterTypes{$TEMPObjectiveCount}{$key}{$keymatch}=1;
 							}
 							$AlternativeCount++;
 						}
 						$ElementCount++;
 					}
 				}
-				#@dmtemp = keys %Filters{$ObjectiveCount};
-				#if($DEBUG) { LogMsg("objective: " . @dmtemp . "\n"); }
+				foreach $eventid (keys(%temp_EventsON)) {
+					$EventsON{$eventid} = $temp_EventsON{$eventid};
+					delete $temp_EventsON{$eventid};
+				}
 
 				$ObjectiveCount++;
 			}
@@ -675,8 +725,18 @@ sub SetAudit() {
 		`/sbin/auditctl -e 1`;
 
 		foreach $eventid (keys(%EventsON)) {
-			if($DEBUG) { LogMsg("Turning on event $eventid\n"); }
-			`/sbin/auditctl -a entry,always -S $eventid`;
+			if ($EventsON{$eventid} == ($SUCCESS + $FAILURE)) {
+				if($DEBUG) { LogMsg("Turning on event $eventid"); }
+				`/sbin/auditctl -a entry,always -S $eventid`;
+			} elsif ($EventsON{$eventid} == $SUCCESS) {
+				if($DEBUG) { LogMsg("Turning on event $eventid (SUCCESS only)"); }
+				`/sbin/auditctl -a exit,always -S $eventid -F success=1`;
+			} elsif ($EventsON{$eventid} == $FAILURE) {
+				if($DEBUG) { LogMsg("Turning on event $eventid (FAILURE only)"); }
+				`/sbin/auditctl -a exit,always -S $eventid -F success=0`;
+			} else {
+				if($DEBUG) { LogMsg("fail $eventid"); }
+			}
 		}
 	}
 }
