@@ -105,11 +105,11 @@ sub WebServerSetup() {
 		if($accesskey) {
 			ReadConfig();
 			# Authentication required.
-			if($request =~ /Authorization: Basic /) {
+			if($request =~ /Authorization: Basic ([^\r]*)\r/) {
 				# Get the supplied password.
 				# md5_hex("blah");
-				($null,$suppliedauth)=split(/Authorization: Basic /,$request);
-				$suppliedauth =~ s/,.*//;
+				$suppliedauth = $1;
+				$suppliedauth =~ s/[\n,].*//;
 				$suppliedauth = base64decode($suppliedauth);
 				($susername,$spassword)=split(/:/,$suppliedauth);
 				$md5password=md5_hex($spassword);
@@ -493,24 +493,24 @@ sub HandleWebPages() {
 		}
 		if($url =~ /\?NISPOM=1$/) {
 			delete($Config{"OBJECTIVES"});
-			$Config{"OBJECTIVES"}{0}="	criticality=3	event=$PROCESS_EVENTS	uid=*,root";
-			$Config{"OBJECTIVES"}{1}="	criticality=3	event=($ADMIN_EVENTS)	uid=*,root";
-			$Config{"OBJECTIVES"}{2}="	criticality=0	event=($OPEN_EVENTS)	name=/etc/*";
+			$Config{"OBJECTIVES"}{0}="	criticality=0	event=($OPEN_EVENTS)	name=/etc/*";
+			$Config{"OBJECTIVES"}{1}="	criticality=3	event=$PROCESS_EVENTS	uid=*,root";
+			$Config{"OBJECTIVES"}{2}="	criticality=3	event=($ADMIN_EVENTS)	uid=*,root";
 			$Config{"OBJECTIVES"}{3}="	criticality=0	event=$PROCESS_EVENTS	exe=*passwd*";
 			$rc=WriteConfig();
 		} elsif($url =~ /\?SOX=1$/) {
 			delete($Config{"OBJECTIVES"});
-			$Config{"OBJECTIVES"}{0}="	criticality=0	event=$PROCESS_EVENTS	uid=*,root";
-			$Config{"OBJECTIVES"}{1}="	criticality=3	event=($ADMIN_EVENTS)	uid=*,root";
-			$Config{"OBJECTIVES"}{2}="	criticality=0	event=($OPEN_EVENTS)	name=/etc/*";
+			$Config{"OBJECTIVES"}{0}="	criticality=0	event=($OPEN_EVENTS)	name=/etc/*";
+			$Config{"OBJECTIVES"}{1}="	criticality=0	event=$PROCESS_EVENTS	uid=*,root";
+			$Config{"OBJECTIVES"}{2}="	criticality=3	event=($ADMIN_EVENTS)	uid=*,root";
 			$Config{"OBJECTIVES"}{3}="	criticality=0	event=$PROCESS_EVENTS	exe=*passwd*";
 			$Config{"OBJECTIVES"}{4}="	criticality=0	event=$NETWORK_EVENTS";
 			$rc=WriteConfig();
 		} elsif($url  =~ /\?PCI=1$/) {
 			delete($Config{"OBJECTIVES"});
-			$Config{"OBJECTIVES"}{0}="	criticality=3	event=$PROCESS_EVENTS	uid=*,root";
-			$Config{"OBJECTIVES"}{1}="	criticality=3	event=($ADMIN_EVENTS)	uid=*,root";
-			$Config{"OBJECTIVES"}{2}="	criticality=0	event=($OPEN_EVENTS)	name=/etc/*";
+			$Config{"OBJECTIVES"}{0}="	criticality=0	event=($OPEN_EVENTS)	name=/etc/*";
+			$Config{"OBJECTIVES"}{1}="	criticality=3	event=$PROCESS_EVENTS	uid=*,root";
+			$Config{"OBJECTIVES"}{2}="	criticality=3	event=($ADMIN_EVENTS)	uid=*,root";
 			$Config{"OBJECTIVES"}{3}="	criticality=0	event=$PROCESS_EVENTS	exe=*passwd*";
 			$Config{"OBJECTIVES"}{4}="	criticality=0	event=$NETWORK_EVENTS";
 			$rc=WriteConfig();
@@ -547,8 +547,14 @@ sub HandleWebPages() {
 					$type++;
 				} elsif($type == 2) {
 					$match=$component;
-					$ObjectiveMatch{$token}=$match;
-					$ObjectiveCompare{$token}=$comp;
+					if ($token =~ /(criticality|uid|return|event)/) {
+						$ObjectiveMatch{$token}=$match;
+						$ObjectiveCompare{$token}=$comp;
+					} else {
+						$elements=keys(%{$ObjectiveMatch{$token}});
+						$ObjectiveMatch{$token}{$elements}=$match;
+						$ObjectiveCompare{$token}{$elements}=$comp;
+					}
 					$type=0;
 				}
 			}
@@ -639,19 +645,21 @@ sub HandleWebPages() {
 			$mcount=0;
 			foreach $matchtype (keys(%ObjectiveMatch)) {
 				$token=$matchtype;
-				$value=$ObjectiveMatch{$matchtype};
-				$cmp=$ObjectiveCompare{$matchtype};
-				print $theclient "<table border=0><tr><td>";
-				print $theclient "<input type=text id=\"ObjMatchToken:$mcount-$count\" name=\"ObjMatchToken:$mcount-$count\" size=13 style=\"background: #eeeeff;\" value=\"$token\">\n";
-				print $theclient "</td><td>";
-				print $theclient "<select name=\"ObjMatchCMP:$mcount-$count\" size=1 style=\"background: #eeeeff;\">";
-				print $theclient "<option value=\"=\"" . ($cmp ne "!="?" selected":"")  . ">=</option>";
-				print $theclient "<option value=\"!=\"" . ($cmp eq "!="?" selected":"") . ">!=</option>";
-				print $theclient "</select></td>\n";
-				print $theclient "</tr><tr><td colspan=2>";
-				print $theclient "<input type=text id=\"ObjMatch:$mcount-$count\" name=\"ObjMatch:$mcount-$count\" size=20 value=\"$value\" style=\"background: #eeeeff;\">";
-				print $theclient "</td></tr></table>";
-				$mcount++;
+				foreach $elem (keys(%{$ObjectiveMatch{$token}})) {
+					$value=$ObjectiveMatch{$matchtype}{$elem};
+					$cmp=$ObjectiveCompare{$matchtype}{$elem};
+					print $theclient "<table border=0><tr><td>";
+					print $theclient "<input type=text id=\"ObjMatchToken:$mcount-$count\" name=\"ObjMatchToken:$mcount-$count\" size=13 style=\"background: #eeeeff;\" value=\"$token\">\n";
+					print $theclient "</td><td>";
+					print $theclient "<select name=\"ObjMatchCMP:$mcount-$count\" size=1 style=\"background: #eeeeff;\">";
+					print $theclient "<option value=\"=\"" . ($cmp ne "!="?" selected":"")  . ">=</option>";
+					print $theclient "<option value=\"!=\"" . ($cmp eq "!="?" selected":"") . ">!=</option>";
+					print $theclient "</select></td>\n";
+					print $theclient "</tr><tr><td colspan=2>";
+					print $theclient "<input type=text id=\"ObjMatch:$mcount-$count\" name=\"ObjMatch:$mcount-$count\" size=20 value=\"$value\" style=\"background: #eeeeff;\">";
+					print $theclient "</td></tr></table>";
+					$mcount++;
+				}
 			}
 
 			$match="AddMatch=$count";
